@@ -12,8 +12,12 @@
 #include <string.h>
 #include <errno.h>
 
+#include "utils/malloc.h"
 #include "plot/plotInt.h"
 
+/* C99 compat */
+#include "textio/textio.h"
+
 #ifdef VERSATEC
 
 extern int PlotRTLCompress();
@@ -43,7 +47,7 @@ PlotHPRTLHeader(width, height, density, hpfile)
     fprintf(hpfile, "\033*b2M");	   /* Mode 2 row compression */
     fprintf(hpfile, "\033*t%dR", density); /* Plotting density in DPI. */
     fprintf(hpfile, "\033*r0A");	   /* Start raster data. */
-} 
+}
 
 /*
  * ----------------------------------------------------------------------------
@@ -55,7 +59,7 @@ PlotHPRTLHeader(width, height, density, hpfile)
  * ----------------------------------------------------------------------------
  */
 
-#define	LABEL_SPACER 200	/* Height of the label area, in pixels	  */ 
+#define	LABEL_SPACER 200	/* Height of the label area, in pixels	  */
 #define IN_TO_HPGL  1016	/* HPGL2 coordinates are in 0.025mm units */
 #define THIN_MARGIN   40	/* thin spacer (1mm) in HPGL2 coordinates */
 
@@ -101,7 +105,6 @@ PlotHPGL2Header(width, height, density, scale, hpfile)
     /* Due to this oversight, it is necessary to build the CMY color	*/
     /* table by hand by reversing the RGB color table.			*/
 
-    /*
     fwrite("\033*v6W\0\0\3\1\1\1", 11, 1, hpfile);
     fprintf(hpfile, "\033*v255a255b255c0I\n");
     fprintf(hpfile, "\033*v0a255b255c1I\n");
@@ -111,10 +114,6 @@ PlotHPGL2Header(width, height, density, scale, hpfile)
     fprintf(hpfile, "\033*v0a255b0c5I\n");
     fprintf(hpfile, "\033*v255a0b0c6I\n");
     fprintf(hpfile, "\033*v0a0b0c7I\n");
-    */
-
-    /* Try simple fixed palette, 3-plane, CMY */
-    fprintf(hpfile, "\033*r-3U\n");
 
     fprintf(hpfile, "\033*r%dS", width);   /* Set the image width in pixels. */
     fprintf(hpfile, "\033*r%dT", height);  /* Set the image height in pixels.*/
@@ -171,22 +170,22 @@ PlotHPGL2Trailer(hpfile)
  * packbits for me.  I was sorely disappointed to find a large (225 lines)
  * routine that will do this, but replys heavily on the internal data
  * structures used by the rest of the libtiff library.
- * 
+ *
  * So, I wrote my own.  Here is a much simpler pair of routines to do the same
  * thing.  These were tested on both the PaintJet-300XL and DesignJet 650C,
  * having been compiled using an HP9000s800.  If anyone is actually interested
  * in the driver program that converts a PPM image for use on these printers, I
  * could probably post that as well.
- * 
+ *
  * Noel (ngorelic@speclab.cr.usgs.gov)
  *-----------------------------------------------------------------------
  */
 
-/* 
+/*
  *-----------------------------------------------------------------------
  *
  * PlotRTLCompress --
- *  
+ *
  * This routine encodes a string using the TIFF packbits encoding
  * scheme.  This encoding method is used by several HP peripherals to
  * reduce the size of incoming raster images.  Both routines convert s1
@@ -200,7 +199,7 @@ PlotHPGL2Trailer(hpfile)
  *
  * Side effects:
  *	Output data placed in s2, which must be large enough to hold it.
- * 
+ *
  *-----------------------------------------------------------------------
  */
 
@@ -308,7 +307,7 @@ PlotDumpHPRTL(hpfile, kRaster, cRaster, mRaster, yRaster)
 
     ipl = kRaster->ras_intsPerLine;
     bpl = kRaster->ras_bytesPerLine;
-    
+
     c = cRaster->ras_bits;
     m = mRaster->ras_bits;
     y = yRaster->ras_bits;
@@ -322,9 +321,12 @@ PlotDumpHPRTL(hpfile, kRaster, cRaster, mRaster, yRaster)
 	/* Merge the black plane into C, M, and Y */
 	for (count = 0; count < ipl; count++)
 	{
-	    *c++ = (*c | *k);
-	    *m++ = (*m | *k);
-	    *y++ = (*y | *k);
+	    *c = (*c | *k);
+	    *m = (*m | *k);
+	    *y = (*y | *k);
+	    c++;
+	    m++;
+	    y++;
 	    k++;
 	}
 
