@@ -1,15 +1,15 @@
 /* geometry.c --
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  * This file contains a bunch of utility routines for manipulating
@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/geometry.c,v 1.2 2008/02/07 05:44:22 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/geometry.c,v 1.5 2008/12/11 14:11:46 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -75,9 +75,10 @@ global int GeoOppositePos[] =
  */
 
 global Rect  GeoNullRect = { 0, 0, 0, 0 };
+global Rect  GeoInvertedRect = { 0, 0, -1, -1 };
 global Point GeoOrigin = { 0, 0 };
 
-
+
 /*-------------------------------------------------------------------
  *	GeoTransPoint --
  *	Transforms a point from one coordinate system to another.
@@ -142,7 +143,10 @@ GeoTransPointDelta(t, p1, p2)
 /*
  *-------------------------------------------------------------------
  *  Determine how an angle changes through transformation via a
- *  tranformation matrix.
+ *  tranformation matrix.  Expects the transformations to be in
+ *  multiples of 90 degrees, plus possible flipping.  Expects an
+ *  angle between 0 and 360 and returns an angle between 0 and
+ *  360.
  *-------------------------------------------------------------------
  */
 
@@ -151,18 +155,40 @@ GeoTransAngle(t, a)
     Transform *t;	/* Transformation matrix */
     int a;		/* Angle to transform */
 {
-    double rads;
-    int ctma;		/* Angle change due to transformation */
+    bool flip = FALSE;
+    int asave = a;
 
-    rads = atan2((double)t->t_d, (double)t->t_a);
-    ctma = (int)(rads / 0.01745329252);  /* pi / 180 */
-    a += ctma;
+    /* Rotate according to the standard transforms */
+
+    if (t->t_a == 0 && t->t_e == 0)
+    {
+	if (t->t_b > 0)
+	    a += 90;
+	else
+	    a += 270;
+	if (t->t_b == t->t_d)
+	    flip = TRUE;
+    }
+    else
+    {
+	if (t->t_a < 0)
+	    a += 180;
+	if (t->t_a != t->t_e)
+	    flip = TRUE;
+    }
     if (a > 360) a -= 360;
-    else if (a < 0) a += 360;
+    if (flip)
+    {
+	if (asave > 90 && asave < 270)
+	    a = 360 - a;
+	else
+	    a = -a;
+    }
+    if (a < 0) a += 360;
     return a;
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoTransRect --
  *	Transforms a rectangle from one coordinate system to another.
@@ -217,7 +243,7 @@ GeoTransRect(t, r1, r2)
 	r2->r_ytop = y1;
     }
 }
-
+
 /*-------------------------------------------------------------------
  *	GeoTranslateTrans --
  *	Translate a transform by the indicated (x, y) amount.
@@ -244,7 +270,7 @@ GeoTranslateTrans(trans1, x, y, trans2)
     trans2->t_c = trans1->t_c + x;
     trans2->t_f = trans1->t_f + y;
 }
-
+
 /*-------------------------------------------------------------------
  *	GeoTransTranslate --
  *	Transform a translation by the indicated (x, y) amount.
@@ -278,7 +304,7 @@ GeoTransTranslate(x, y, trans1, trans2)
     trans2->t_f = x*trans1->t_d + y*trans1->t_e + trans1->t_f;
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoTransTrans --
  *	This routine transforms a transform.
@@ -306,7 +332,7 @@ GeoTransTrans(first, second, net)
     net->t_f = first->t_c*second->t_d + first->t_f*second->t_e + second->t_f;
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoNameToPos --
  *	Map the name of a position into an integer position parameter.
@@ -417,7 +443,7 @@ GeoNameToPos(name, manhattan, verbose)
     return (pos);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -444,7 +470,7 @@ GeoPosToName(pos)
         case GEO_NORTH:     return("NORTH");
         case GEO_NORTHEAST: return("NORTHEAST");
         case GEO_EAST:      return("EAST");
-        case GEO_SOUTHEAST: return("SOTHEAST");
+        case GEO_SOUTHEAST: return("SOUTHEAST");
         case GEO_SOUTH:     return("SOUTH");
         case GEO_SOUTHWEST: return("SOUTHWEST");
         case GEO_WEST:      return("WEST");
@@ -452,7 +478,7 @@ GeoPosToName(pos)
 	default:            return("*ILLEGAL*");
     }
 }
-
+
 /*-------------------------------------------------------------------
  *	GeoTransPos --
  *	This routine computes the transform of a relative position.
@@ -504,7 +530,65 @@ GeoTransPos(t, pos)
     return pos;
 }
 
-
+/*-------------------------------------------------------------------
+ *	GeoTransOrient --
+ *	This routine returns the orientation corresponding to a transform.
+ *
+ *	Results:
+ *	The return value is an orientation as defined by the enumeration
+ *	below (which is also used by the LEF read routine).  It has to
+ *	agree with the enumeration used by dbOrientUseFunc.
+ *
+ *	Side Effects:	None.
+ *-------------------------------------------------------------------
+ */
+
+enum def_orient {ORIENT_NORTH, ORIENT_SOUTH, ORIENT_EAST, ORIENT_WEST,
+        ORIENT_FLIPPED_NORTH, ORIENT_FLIPPED_SOUTH, ORIENT_FLIPPED_EAST,
+        ORIENT_FLIPPED_WEST};
+
+int
+GeoTransOrient(t)
+    Transform *t;		/* Transform to be applied. */
+
+{
+    int pidx;
+
+    if ((t->t_b == 0) && (t->t_d == 0))
+    {
+	pidx = ((t->t_a) > 0) ? 1 : 0;
+	pidx += ((t->t_e) > 0) ? 2 : 0;
+
+	switch (pidx) {
+	    case 0:
+		return ORIENT_SOUTH;
+	    case 1:
+		return ORIENT_FLIPPED_SOUTH;
+	    case 2:
+		return ORIENT_FLIPPED_NORTH;
+	    case 3:
+		return ORIENT_NORTH;
+	}
+    }
+    else if ((t->t_a == 0) && (t->t_e == 0))
+    {
+	pidx = ((t->t_b) > 0) ? 1 : 0;
+	pidx += ((t->t_d) > 0) ? 2 : 0;
+
+	switch (pidx) {
+	    case 0:
+		return ORIENT_FLIPPED_EAST;
+	    case 1:
+		return ORIENT_EAST;
+	    case 2:
+		return ORIENT_WEST;
+	    case 3:
+		return ORIENT_FLIPPED_WEST;
+	}
+    }
+}
+
+
 /*-------------------------------------------------------------------
  *	GeoInvertTrans --
  *	This routine computes the inverse of a transform.
@@ -538,7 +622,7 @@ GeoInvertTrans(t, inverse)
     GeoTransTrans(&t2, &t3, inverse);
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoInclude --
  *	This routine includes one rectangle into another by expanding
@@ -565,7 +649,7 @@ GeoInclude(src, dst)
 	*dst = *src;
 	return TRUE;
     }
-    
+
     value = FALSE;
     if (dst->r_xbot > src->r_xbot)
     {
@@ -590,7 +674,7 @@ GeoInclude(src, dst)
     return value;
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoIncludeAll --
  *	This routine includes one rectangle into another by expanding
@@ -623,7 +707,7 @@ GeoIncludeAll(src, dst)
 
     if ((src->r_xbot > src->r_xtop) || (src->r_ybot > src->r_ytop))
 	return FALSE;
-    
+
     value = FALSE;
     if (dst->r_xbot > src->r_xbot)
     {
@@ -648,7 +732,7 @@ GeoIncludeAll(src, dst)
     return value;
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoIncludePoint --
  *	This routine includes a point into a rectangle by expanding
@@ -689,7 +773,25 @@ GeoIncludePoint(src, dst)
     }
 }
 
-
+/*-------------------------------------------------------------------
+ *	GeoIncludeRectInBBox() --
+ *
+ *	Expand bounding box to include rectangle r
+ *
+ *-------------------------------------------------------------------
+ */
+void
+GeoIncludeRectInBBox(r, bbox)
+    Rect *r;
+    Rect *bbox;
+{
+    bbox->r_xbot = MIN(bbox->r_xbot,r->r_xbot);
+    bbox->r_ybot = MIN(bbox->r_ybot,r->r_ybot);
+    bbox->r_xtop = MAX(bbox->r_xtop,r->r_xtop);
+    bbox->r_ytop = MAX(bbox->r_ytop,r->r_ytop);
+}
+
+
 /*-------------------------------------------------------------------
  *	GeoClip --
  *	clips one rectangle against another.
@@ -716,7 +818,7 @@ GeoClip(r, area)
     if (r->r_xtop > area->r_xtop) r->r_xtop = area->r_xtop;
     if (r->r_ytop > area->r_ytop) r->r_ytop = area->r_ytop;
 }
-
+
 /*-------------------------------------------------------------------
  *	GeoClipPoint --
  *	Clips one point against a rectangle, moving the point into
@@ -739,7 +841,7 @@ GeoClipPoint(p, area)
     if (p->p_x > area->r_xtop) p->p_x = area->r_xtop;
     if (p->p_y > area->r_ytop) p->p_y = area->r_ytop;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *	GeoDisjoint --
@@ -790,7 +892,7 @@ GeoDisjoint(area, clipBox, func, cdarg)
      * into one piece that is DEFINITELY outside clipBox, and one
      * piece left to check some more.
      */
-    
+
     /* Top edge of clipBox: */
 
     rArea = *area;
@@ -845,7 +947,7 @@ GeoDummyFunc(box, cdarg)
     return TRUE;
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoCanonicalRect --
  *	Turns a rectangle into a canonical form in which the
@@ -885,7 +987,7 @@ GeoCanonicalRect(r, rnew)
 	rnew->r_ytop = r->r_ytop;
     }
 }
-
+
 /*-------------------------------------------------------------------
  *	GeoScale --
  *
@@ -942,7 +1044,7 @@ GeoScaleTrans(trans1, m, trans2)
     trans2->t_f = trans1->t_f * m;
 }
 
-
+
 /*-------------------------------------------------------------------
  *	GeoRectPointSide --
  *
@@ -971,7 +1073,7 @@ GeoRectPointSide(r, p)
     else
 	return(GEO_CENTER);
 }
-
+
 /*-------------------------------------------------------------------
  *	GeoRectRectSide --
  *
@@ -1003,7 +1105,7 @@ GeoRectRectSide(r0, r1)
     else
 	return(GEO_CENTER);
 }
-
+
 /* ----------------------------------------------------------------------------
  *
  * GeoDecomposeTransform --
@@ -1024,7 +1126,7 @@ GeoRectRectSide(r0, r1)
 void
 GeoDecomposeTransform(t, upsidedown, angle)
     Transform *t;
-    bool *upsidedown;		/* Set to TRUE iff we should flip upsidedown 
+    bool *upsidedown;		/* Set to TRUE iff we should flip upsidedown
 				 * before rotating.
 				 */
     int *angle;			/* Amount to rotate.
@@ -1043,15 +1145,15 @@ GeoDecomposeTransform(t, upsidedown, angle)
     /* Compute rotations and flips. */
     *upsidedown = ((notrans.t_a == 0) ^
 	(notrans.t_b == notrans.t_d) ^ (notrans.t_a == notrans.t_e));
-    if (*upsidedown) 
+    if (*upsidedown)
 	GeoTransTrans(&notrans, &GeoUpsideDownTransform, &rotonly);
     else
 	rotonly = notrans;
     /* Verify no flipping. */
-    ASSERT(rotonly.t_a == rotonly.t_e, "GeoDecomposeTransform");	  
+    ASSERT(rotonly.t_a == rotonly.t_e, "GeoDecomposeTransform");
 
     *angle = 0;
-    if (rotonly.t_b != 0) 
+    if (rotonly.t_b != 0)
     {
 	*angle += 90;
 	if (*upsidedown) *angle += 180;
